/***************************************************************************
 *   Copyright (C) 2008 by Kerry D. Wong
 *   http://www.kerrywong.com
 *
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *     * Neither the name of the author nor the names of its contributors
 *       may be used to endorse or promote products derived from this
 *       software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <iostream>
#include <iomanip>
#include <math.h>

#include "tbb/task_scheduler_init.h"
#include "tbb/tick_count.h"
#include "tbb/blocked_range.h"
#include "tbb/parallel_for.h"
#include "tbb/partitioner.h"

using namespace std;
using namespace tbb;

static const int MAX_SIZE = 1000000;

class prime_single_thread
{
public:
    void run(const blocked_range<size_t> &r)
    {
        bool is_prime = false;

        for (size_t x = r.begin(); x < r.end(); x+=2)
        {
            is_prime = true;

            for (int i = 3; i <= sqrt((double) x); i+=2)
            {
                if (x % i == 0)
                {
                    is_prime = false;
                    continue;
                }
            }

            // Output prime numbers:
//            if (is_prime)
//            {
//                cout << x << endl;
//            }
        }
    }
};

class prime_tbb
{
public:
    void test_prime(int num) const
    {
        bool is_prime = true;

        for (int i = 3; i <= sqrt((double) num); i+=2)
        {
            if (num % i == 0)
            {
                is_prime = false;
                continue;
            }
        }

        // Output prime numbers:
//        if (is_prime)
//        {
//            cout << num << endl;
//        }
    }

    void operator() (const blocked_range<size_t> &r) const
    {
        for (size_t i = r.begin(); i < r.end(); i+=2)
        {
            test_prime(i);
        }
    }

    void run(blocked_range<size_t> r)
    {
        prime_tbb prime_tbb;

        parallel_for(r, prime_tbb);
    }

};

int main()
{
    task_scheduler_init init;
    static tick_count t_start, t_end;

    prime_single_thread p1;
    prime_tbb p2;

    cout.setf(ios::fixed);
    cout.setf(ios::showpoint);
    cout.precision(2);

    //starting from 3, with a granularity of 100.
    blocked_range<size_t> r(3, MAX_SIZE, 100);

    t_start = tick_count::now();
    p1.run(r);
    t_end = tick_count::now();
    cout << (t_end - t_start).seconds() * 1000 << " ms" << endl;

    t_start = tick_count::now();http://www.kerrywong.com
    p2.run(r);
    t_end = tick_count::now();
    cout << (t_end - t_start).seconds() * 1000 << " ms" << endl;

    return 0;
}
